
(function () {
  if (window.DCEPJSBridge) {
    return;
  }

  var messagingIframe;
  var sendMessageQueue = [];
  var receiveMessageQueue = [];
  var messageHandlers = {};

  var CUSTOM_PROTOCOL_SCHEME = "yy";
  var QUEUE_HAS_MESSAGE = "__QUEUE_MESSAGE__/";

  var responseCallbacks = {};
  var uniqueId = 1;

  function _createQueueReadyIframe(doc) {
    messagingIframe = doc.createElement("iframe");
    messagingIframe.style.display = "none";
    doc.documentElement.appendChild(messagingIframe);
  }

  //set default messageHandler
  function init(messageHandler) {
    if (DCEPJSBridge._messageHandler) {
      throw new Error("DCEPJSBridge.init called twice");
    }
    DCEPJSBridge._messageHandler = messageHandler;
    var receivedMessages = receiveMessageQueue;
    receiveMessageQueue = null;
    for (var i = 0; i < receivedMessages.length; i++) {
      _dispatchMessageFromNative(receivedMessages[i]);
    }
  }

  function send(data, responseCallback) {
    _doSend(
      {
        data: data,
      },
      responseCallback
    );
  }

  function registerHandler(handlerName, handler) {
    messageHandlers[handlerName] = handler;
//    console.log("registerHandler","handlerName=",handlerName,"handler=",handler);
//    console.log("registerHandler-messageHandlers=",messageHandlers[handlerName]);
  }

  function removeHandler(handlerName, handler) {
//     console.log("removeHandler","handlerName=",handlerName,"handler=",handler);
    delete messageHandlers[handlerName];
  }

  function callHandler(handlerName, data, responseCallback) {
    _doSend(
      {
        handlerName: handlerName,
        data: data,
      },
      responseCallback
    );
  }

  //sendMessage add message, 触发native处理 sendMessage
  function _doSend(message, responseCallback) {
    if (responseCallback) {
      var callbackId = "cb_" + uniqueId++ + "_" + new Date().getTime();
      responseCallbacks[callbackId] = responseCallback;
      message.callbackId = callbackId;
    }

    sendMessageQueue.push(message);
    messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + "://" + QUEUE_HAS_MESSAGE;
  }

  // 提供给native调用,该函数作用:获取sendMessageQueue返回给native,由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
  function _fetchQueue() {
    var messageQueueString = JSON.stringify(sendMessageQueue);
    sendMessageQueue = [];
    //android can't read directly the return data, so we can reload iframe src to communicate with java
    messagingIframe.src =
      CUSTOM_PROTOCOL_SCHEME +
      "://return/_fetchQueue/" +
      encodeURIComponent(messageQueueString);
  }

  //提供给native使用,
  function _dispatchMessageFromNative(messageJSON) {
    setTimeout(function () {
      var message = JSON.parse(messageJSON);
      var responseCallback;
      //java call finished, now need to call js callback function
//      console.log("dispatchFromNative888:",message);
      if (message.responseId) {
        responseCallback = responseCallbacks[message.responseId];
        if (!responseCallback) {
          return;
        }
        if (typeof message.responseData === 'string' ) {
       try {
         message.responseData = JSON.parse(message.responseData)
         } catch (e) {
          }
        }
        responseCallback(message.responseData);
        delete responseCallbacks[message.responseId];
      } else {
        //直接发送
        if (message.callbackId) {
          var callbackResponseId = message.callbackId;
          responseCallback = function (responseData) {
            _doSend({
              responseId: callbackResponseId,
              responseData: responseData,
            });
          };
        }

        var handler = DCEPJSBridge._messageHandler;
        if (message.handlerName) {
          handler = messageHandlers[message.handlerName];
        }
        //查找指定handler
        try {
//          console.log("_dispatchMessageFromNative-handle",JSON.stringify(messageHandlers));
          handler(message.data, responseCallback);
//          console.log(JSON.stringify(message));
        } catch (exception) {
          if (typeof console != "undefined") {
//            console.log(
//              "DCEPJSBridge: WARNING: javascript handler threw.",
//              message,
//              exception
//            );
          }
        }
      }
    });
  }

  //提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
  function _handleMessageFromNative(messageJSON) {
//    console.log("_handleMessageFromNative111:",messageJSON,receiveMessageQueue);
    if (receiveMessageQueue && receiveMessageQueue.length > 0) {
      receiveMessageQueue.push(messageJSON);
    } else {
//       console.log("MessageFromNative222222:",messageJSON);
      _dispatchMessageFromNative(messageJSON);
    }
  }

  var DCEPJSBridge = (window.DCEPJSBridge = {
    init: init,
    send: send,
    call: h5Call,
    on: registerHandler,
    off: removeHandler,
    _fetchQueue: _fetchQueue,
    _handleMessageFromNative: _handleMessageFromNative,
  });

  // H5侧主动发起调用
  function h5Call(handlerName, data, callback) {
    // 如果方法不需要参数，只有回调函数，简化JS中的调用
    if (arguments.length == 2 && typeof data == "function") {
      responseCallback = data;
      data = null;
    }
    callHandler(handlerName, data, callback);
  }
  // 注册事件等待回调
  function defaultRegisterHandler() {
    // 注册pause
    registerHandler("pause", function () {
      triggerEventHandler("pause");
    });
    // 注册resume
    registerHandler("resume", function() {
      triggerEventHandler("resume");
     });
  }

  // 触发指定事件
  function triggerEventHandler(eventName) {
    var nativeEvent = doc.createEvent("Events");
    nativeEvent.initEvent(eventName);
    doc.dispatchEvent(nativeEvent);
  }

  var doc = document;
  _createQueueReadyIframe(doc);
  var readyEvent = doc.createEvent("Events");
  readyEvent.initEvent("DCEPJSBridgeReady");
  readyEvent.bridge = DCEPJSBridge;
  doc.dispatchEvent(readyEvent);
  // ready 之后自动注册事件 等待native通知后再发射对应事件
  defaultRegisterHandler();
})();
